"""
Orchestration of Undeploy
"""
import xmltodict
from .verify import Verify


class Undeploy:
    """
    Parameters
    ----------
    obj : object
        Thousandeyes Class Instance
    Returns
    -------
    bool
        If successful or failed
    """

    @staticmethod
    def stop(obj):
        """ Stop Thousand Eyes Agent on device """
        rpc = f"""
        <app-hosting xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-rpc">
            <stop>
                <appid>{obj.cfg.appid}</appid>
            </stop>
        </app-hosting>
        """
        data = xmltodict.parse(obj.device_api.rpc(rpc=rpc))
        return Verify.app_status_undeploy(data=data, appid=obj.cfg.appid)

    @staticmethod
    def deactivate(obj):
        """ Deactivate Thousand Eyes Agent on device """
        rpc = f"""
        <app-hosting xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-rpc">
            <deactivate>
                <appid>{obj.cfg.appid}</appid>
            </deactivate>
        </app-hosting>
        """
        data = xmltodict.parse(obj.device_api.rpc(rpc=rpc))
        return Verify.app_status_undeploy(data=data, appid=obj.cfg.appid)

    @staticmethod
    def uninstall(obj):
        """ Uninstall Thousand Eyes Agent on device """
        rpc = f"""
        <app-hosting xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-rpc">
            <uninstall>
                <appid>{obj.cfg.appid}</appid>
            </uninstall>
        </app-hosting>
        """
        data = xmltodict.parse(obj.device_api.rpc(rpc=rpc))
        return Verify.app_status_undeploy(data=data, appid=obj.cfg.appid)

    @staticmethod
    def config(obj):
        """ Removing Application Hosting config on device """
        config = f"""
        <config xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
        <app-hosting-cfg-data xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-app-hosting-cfg">
            <apps operation="delete">
                <app>
                    <application-name>{obj.cfg.appid}</application-name>
                    <application-network-resource>
                        <appintf-vlan-mode>appintf-trunk</appintf-vlan-mode>
                    </application-network-resource>
                    <appintf-vlan-rules>
                        <appintf-vlan-rule>
                            <vlan-id>{obj.cfg.vlan}</vlan-id>
                            <guest-interface>0</guest-interface>
                        </appintf-vlan-rule>
                    </appintf-vlan-rules>
                    <docker-resource>true</docker-resource>
                    <run-optss>
                        <run-opts>
                            <line-index>1</line-index>
                            <line-run-opts>-e TEAGENT_ACCOUNT_TOKEN={obj.cfg.token}</line-run-opts>
                        </run-opts>
                    </run-optss>
                    <prepend-pkg-opts>true</prepend-pkg-opts>
                </app>
            </apps>
        </app-hosting-cfg-data>
        <native xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-native">
            <interface>
            <AppGigabitEthernet>
                <name>1/0/1</name>
            </AppGigabitEthernet>
            </interface>
        </native>
        </config>
        """
        return obj.device_api.config(config=config)

    @staticmethod
    def iox(obj):
        """ Disable IOX Service on device """
        config = """
            <config xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
                <native xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-native">
                <iox operation="delete">
                </iox>
                </native>
            </config>
            """
        return obj.device_api.config(config=config)



"""
Orchestration of Deploy
"""
import time
import xmltodict
from .verify import Verify


class Deploy:
    """
    Parameters
    ----------
    obj : object
        Thousandeyes Class Instance
    Returns
    -------
    bool
        If successful or failed
    """

    @staticmethod
    def hardware(obj):
        """ Check hardware on device """
        __filter = """
        <filter xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
            <device-hardware-data xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-device-hardware-oper">
            <device-hardware>
                <device-inventory>
                <part-number>
                </part-number>
                </device-inventory>
            </device-hardware>
            </device-hardware-data>
        </filter>
        """
        data = xmltodict.parse(obj.device_api.get(filter=__filter))
        return Verify.hardware(data=data)

    @staticmethod
    def subscription(obj):
        """ Check version on devicee """
        __filter = """
        <filter xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
            <licensing xmlns="http://cisco.com/ns/yang/cisco-smart-license">
            <state>
            <state-info>
            <udi>
                <pid>
                </pid>
            </udi>
            <usage>
                <license-name>
                </license-name>
            </usage>
            </state-info>
            </state>
            </licensing>
        </filter>
        """
        data = xmltodict.parse(obj.device_api.get(filter=__filter))
        return Verify.subscription(data=data)

    @staticmethod
    def version(obj):
        """ Check version on devicee """
        __filter = """
        <filter xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
            <install-oper-data xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-install-oper">
            <install-location-information>
            <install-version-state-info>
            <version>
            </version>
            </install-version-state-info>
            </install-location-information>
            </install-oper-data>
        </filter>
        """
        data = xmltodict.parse(obj.device_api.get(filter=__filter))
        return Verify.version(data=data)

    @staticmethod
    def iox(obj):
        """ Check IOX sevice on device """
        __filter = """
        <filter xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
            <native xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-native">
            <iox>
            </iox>
            </native>
        </filter>
        """
        data = xmltodict.parse(obj.device_api.get(filter=__filter))
        if Verify.iox(data=data) is False:
            config = """
            <config xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
                <native xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-native">
                <iox>
                </iox>
                </native>
            </config>
            """
            if obj.device_api.config(config=config) is True:
                # Wait until IOX is ready
                time.sleep(300)
                return True
            else:
                return False
        else:
            return True

    @staticmethod
    def apps(obj):
        """ Check existing apps hosted on device """
        __filter = """
        <filter xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
            <app-hosting-oper-data xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-app-hosting-oper">
            <app>
            </app>
            </app-hosting-oper-data>
        </filter>
        """
        data = xmltodict.parse(obj.device_api.get(filter=__filter))
        return Verify.apps(data=data)

    @staticmethod
    def install(obj):
        """ Install Thousand Eyes Agent on device """
        rpc = f"""
        <app-hosting xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-rpc">
            <install>
                <appid>{obj.cfg.appid}</appid>
                <package>{obj.cfg.download_url}</package>
            </install>
        </app-hosting>
        """
        data = xmltodict.parse(obj.device_api.rpc(rpc=rpc))
        if Verify.app_status_deploy(data=data, appid=obj.cfg.appid) is True:
            max_retry = 10
            retry = 0
            while True:
                if Deploy.apps(obj) is False or retry == max_retry:
                    return True
                retry += 1
                time.sleep(15)
            return False
        else:
            return False

    @staticmethod
    def config(obj):
        """ Configure Thousand Eyes Agent on device """
        config = f"""
        <config xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
        <app-hosting-cfg-data xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-app-hosting-cfg">
            <apps operation="replace">
                <app>
                    <application-name>{obj.cfg.appid}</application-name>
                    <application-network-resource>
                        <appintf-vlan-mode>appintf-trunk</appintf-vlan-mode>
                    </application-network-resource>
                    <appintf-vlan-rules>
                        <appintf-vlan-rule>
                            <vlan-id>{obj.vlan}</vlan-id>
                            <guest-interface>0</guest-interface>
                        </appintf-vlan-rule>
                    </appintf-vlan-rules>
                    <docker-resource>true</docker-resource>
                    <run-optss>
                        <run-opts>
                            <line-index>1</line-index>
                            <line-run-opts>-e TEAGENT_ACCOUNT_TOKEN={obj.cfg.token}</line-run-opts>
                        </run-opts>
                    </run-optss>
                    <prepend-pkg-opts>true</prepend-pkg-opts>
                </app>
            </apps>
        </app-hosting-cfg-data>
        <native xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-native">
            <interface>
            <AppGigabitEthernet operation="replace">
                <name>1/0/1</name>
                <description>{obj.cfg.appid}</description>
                <switchport>
                <mode xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-switch">
                <trunk xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-switch">
                </trunk>
                </mode>
                <trunk xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-switch">
                <allowed>
                    <vlan>
                        <vlans>{obj.vlan}</vlans>
                    </vlan>
                </allowed>
                </trunk>
                </switchport>
                <macro xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-switch">
                <auto>
                <processing>false</processing>
                </auto>
                </macro>
            </AppGigabitEthernet>
            </interface>
        </native>
        </config>
        """
        return obj.device_api.config(config=config)

    @staticmethod
    def activate(obj):
        """ Activate Thousand Eyes Agent on device """
        rpc = f"""
        <app-hosting xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-rpc">
            <activate>
                <appid>{obj.cfg.appid}</appid>
            </activate>
        </app-hosting>
        """
        data = xmltodict.parse(obj.device_api.rpc(rpc=rpc))
        return Verify.app_status_deploy(data=data, appid=obj.cfg.appid)

    @staticmethod
    def start(obj):
        """ Start Thousand Eyes Agent on device """
        rpc = f"""
        <app-hosting xmlns="http://cisco.com/ns/yang/Cisco-IOS-XE-rpc">
            <start>
                <appid>{obj.cfg.appid}</appid>
            </start>
        </app-hosting>
        """
        data = xmltodict.parse(obj.device_api.rpc(rpc=rpc))
        return Verify.app_status_deploy(data=data, appid=obj.cfg.appid)



#!/usr/bin/env python
"""cat9kthousandeyesctl Console Script.

Copyright (c) 2021 Cisco and/or its affiliates.

This software is licensed to you under the terms of the Cisco Sample
Code License, Version 1.1 (the "License"). You may obtain a copy of the
License at

               https://developer.cisco.com/docs/licenses

All use of the material herein must be in accordance with the terms of
the License. All rights not expressly granted by the License are
reserved. Unless required by applicable law or agreed to separately in
writing, software distributed under the License is distributed on an "AS
IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
or implied.

"""
import logging
import sys
import click
from .thousandeyes import Thousandeyes
from . import Metadata


@click.group()
@click.version_option(Metadata.version)
@click.option("--debug", is_flag=True, help="Enable logging")
@click.pass_context
def cli(ctx, debug):
    """
    Manage Thousand Eyes Agent on Catalyst 9000
    """
    ctx.ensure_object(dict)
    ctx.obj["debug"] = debug
    pass


@cli.command()
@click.pass_context
def interactive(ctx):
    """
    Interactive TTY mode
    """
    if ctx.obj["debug"] is True:
        enable_debug()
    print("Interactive mode")
    config = {}
    __hosts = click.prompt("Hosts", type=str).replace(" ", "")
    config["hosts"] = __hosts.split(",")
    config["username"] = click.prompt("Username", type=str)
    config["password"] = click.prompt("Password", type=str, hide_input=True)
    config["port"] = click.prompt("Port", default=830, type=int)
    config["timeout"] = click.prompt("Timeout", default=300, type=int)
    config["vlan"] = click.prompt("VLAN", type=int)
    config["vlan"] = click.prompt("VLAN", type=int)
    config["token"] = click.prompt("Thousand Eyes Token", type=str)
    config["download_url"] = click.prompt(
        "Download URL",
        default="https://downloads.thousandeyes.com/enterprise-agent/thousandeyes-enterprise-agent-3.0.cat9k.tar",
    )
    config["appid"] = click.prompt("AppId", default="thousandeyes_enterprise_agent")
    cfg = Thousandeyes.Configs.interactive(config=config)
    mode = click.prompt("Deploy or Undeploy?", type=str, default="deploy")
    proccess = Thousandeyes(cfg)
    for host in cfg.hosts:
        if "undeploy" in mode.lower():
            proccess.undeploy(host)
        if "deploy" in mode.lower():
            proccess.deploy(host, vlan=cfg.vlan)


@cli.command()
@click.option("--config", "-c", required=True, type=str, help="Config")
@click.pass_context
def deploy(ctx, config):
    """
    Deploy Thousand Eyes Agent
    """
    if ctx.obj["debug"] is True:
        enable_debug()
    print("Deploying Thousand Eyes Agents")
    cfg = Thousandeyes.Configs.load(config=config)
    proccess = Thousandeyes(cfg)

    for host, override in cfg.hosts.items():
        if isinstance(override, dict):
            if "vlan" in override:
                try:
                    proccess.deploy(host, vlan=override["vlan"])
                except Exception as error_msg:
                    print(f"Can't connect to {host} - {error_msg}")
                    pass
        else:
            try:
                proccess.deploy(host, vlan=cfg.vlan)
            except Exception as error_msg:
                print(f"Can't connect to {host} - {error_msg}")
                pass


@cli.command()
@click.option("--config", "-c", required=True, type=str, help="Config")
@click.pass_context
def undeploy(ctx, config):
    """
    Remove Thousand Eyes Agent
    """
    if ctx.obj["debug"] is True:
        enable_debug()
    print("Undeploying Thousand Eyes Agents")
    cfg = Thousandeyes.Configs.load(config=config)
    proccess = Thousandeyes(cfg)
    for host in cfg.hosts:
        try:
            proccess.undeploy(host)
        except Exception as error_msg:
            print(f"Can't connect to {host} - {error_msg}")
            pass


def enable_debug():
    print("Debug enabled")
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s %(levelname)s: %(message)s",
        stream=sys.stdout,
    )


if __name__ == "__main__":
    cli()



"""
Device API using Netconf
"""
from ncclient import manager
from lxml import etree


class Netconf:
    """
    Methods
    ----------
    _connect : obj
        Establish Netconf over SSH
    get: xml
        Get config
    config: xml
        Edit config
    rpc: xml
        RPC commands
    """

    def __init__(self, host=None, port=830, timeout=600, username=None, password=None):
        """
        Parameters
        ----------
        host : str
            Device host
        port: int
            Device port
        timeout: int
            Netconf port
        username: str
            Device access
        password: str
            Device access
        """
        self.host = host
        self.port = port
        self.timeout = timeout
        self.username = username
        self.password = password

    def _connect(self):
        """
        Parameters
        ----------
        host : str
            Device host
        port: int
            Device port
        timeout: int
            Netconf port
        username: str
            Device access
        password: str
            Device access

        Returns
        -------
        obj
            Netconf connection object
        """
        return manager.connect(
            host=self.host,
            port=self.port,
            username=self.username,
            password=self.password,
            hostkey_verify=False,
            timeout=self.timeout,
        )

    def get(self, filter=None):
        """
        Parameters
        ----------
        filter : str
            Netconf XML XPath
        Returns
        -------
        c: str
            XML
        """
        with self._connect() as m:
            c = m.get(filter).data
        return etree.tostring(c, encoding="unicode", pretty_print=True)

    def config(self, config):
        """
        Parameters
        ----------
        config : str
            Netconf XML
        Returns
        -------
        c: bool
            Successful or failed
        """
        with self._connect() as m:
            c = m.edit_config(target="running", config=config)
            if c.ok:
                return True
            else:
                print(c.error)
                return False

    def rpc(self, rpc):
        """
        Parameters
        ----------
        rpc : str
            Netconf XML RPC
        Returns
        -------
        c: str
            XML
        """
        with self._connect() as m:
            c = m.dispatch(etree.fromstring(rpc))
            return str(c)



"""
Validate data from Catalyst 9000
"""
from .configs import Supported


class Verify:
    """
    Parameters
    ----------
    data : dict (xml)
        XML data
    Returns
    -------
    bool
        If successful or failed
    """

    @staticmethod
    def hardware(data):
        """ Check hardware on device """
        try:
            data["data"]["device-hardware-data"]["device-hardware"]["device-inventory"]
        except Exception:
            raise Exception(
                "Can't determinate the Catalyst 9000 Hardware of the device"
            )

        for pid in data["data"]["device-hardware-data"]["device-hardware"][
            "device-inventory"
        ]:
            for h in Supported.hardware:
                if h in str(pid["part-number"]):
                    return True
        return False

    @staticmethod
    def subscription(data):
        """ Check DNA Subscription on device """
        try:
            data["data"]["licensing"]["state"]["state-info"]["usage"]
        except Exception:
            raise Exception("Can't determinate the DNA Subscription of the device")

        for sub in data["data"]["licensing"]["state"]["state-info"]["usage"]:
            for s in Supported.subscriptions:
                if s in str(sub["license-name"]):
                    return True
        return False

    @staticmethod
    def version(data):
        """ Check version on devicee """
        try:
            __version = data["data"]["install-oper-data"][
                "install-location-information"
            ]["install-version-state-info"]["version"]
        except Exception:
            raise Exception("Can't determinate the IOS-XE version of the device")

        for v in Supported.versions:
            if v in __version:
                return True
        return False

    @staticmethod
    def app_status_deploy(data, appid):
        """ Check deploy status """
        catch_succcess = [
            "Installing package",
            "RUNNING",
            "ACTIVATED",
        ]
        if "#text" in data["rpc-reply"]["result"]:
            if appid in data["rpc-reply"]["result"]["#text"]:
                for status in catch_succcess:
                    if status.lower() in data["rpc-reply"]["result"]["#text"].lower():
                        return True
        return False

    @staticmethod
    def app_status_undeploy(data, appid):
        """ Check undeploy status """
        catch_succcess = [
            "STOPPED",
            "DEPLOYED",
            "Uninstalling",
        ]
        if "#text" in data["rpc-reply"]["result"]:
            if appid in data["rpc-reply"]["result"]["#text"]:
                for status in catch_succcess:
                    if status.lower() in data["rpc-reply"]["result"]["#text"].lower():
                        return True
        return False

    @staticmethod
    def apps(data):
        """ Check existing apps hosted on device """
        if "app-hosting-oper-data" in data["data"]:
            # Applications already installed on Host
            return False
        return True

    @staticmethod
    def iox(data):
        """ Check IOX sevice on device """
        if "native" in data["data"]:
            for k in data["data"]["native"].keys():
                # IOX already configured on Host
                if k == "iox":
                    return True
        return False



"""
Orchestration of Thousand Eyes agents on Catalyst 9000
"""
from tqdm import tqdm
from .netconf import Netconf
from .configs import Configs
from .deploy import Deploy
from .undeploy import Undeploy


class Thousandeyes:
    """ Class for Deploy and Undeploy """

    class Configs(Configs):
        """ Load config """

        pass

    def __init__(self, cfg):
        self.stages = {
            "deploy": [
                "check hardware",
                "check subscription",
                "check ios-xe version",
                "check iox",
                "check existing apps",
                f"install {cfg.appid}",
                f"config {cfg.appid}",
                f"activate {cfg.appid}",
                f"start {cfg.appid}",
            ],
            "undeploy": [
                f"stop {cfg.appid}",
                f"deactivate {cfg.appid}",
                f"uninstall {cfg.appid}",
                f"remove config {cfg.appid}",
                "disable iox",
            ],
        }
        self.cfg = cfg

    def deploy(self, host, vlan):
        """
        Parameters
        ----------
        host : str
            Catalyst 9000 host
        vlan: int
            Thousand Eyes Agent VLAN
        Returns
        -------
        bool
            If successful or failed
        """

        self.device_api = Netconf(
            host=host,
            username=self.cfg.username,
            password=self.cfg.password,
            port=self.cfg.port,
            timeout=self.cfg.timeout,
        )
        self.host = host
        self.vlan = vlan
        self.mode = "deploy"

        self.pbar = tqdm(self.stages[self.mode], desc=f"{self.host}: connecting")
        for stage in self.pbar:
            self.progressbar(stage=stage)

            if "hardware" in stage:
                # Stage 0
                if Deploy.hardware(self) is not True:
                    self.progressbar(
                        error=True, msg="Non-supported Cisco Catalyst Hardware"
                    )
                    return False

            if "subscription" in stage:
                # Stage 1
                if Deploy.subscription(self) is not True:
                    self.progressbar(
                        error=True, msg="Non-supported Cisco DNA Subscription"
                    )
                    return False

            if "ios-xe" in stage:
                # Stage 2
                if Deploy.version(self) is not True:
                    self.progressbar(error=True, msg="Non-supported IOS-XE version")
                    return False

            if "iox" in stage:
                # Stage 3
                if Deploy.iox(self) is not True:
                    self.progressbar(
                        error=True, msg="Couldn't enable IOX (container support)"
                    )
                    return False

            if "apps" in stage:
                # Stage 4
                if Deploy.apps(self) is not True:
                    self.progressbar(
                        error=True,
                        msg="There's an app already configured",
                    )
                    return False

            if "install" in stage:
                # Stage 5
                if Deploy.install(self) is not True:
                    self.progressbar(
                        error=True, msg=f"Couldn't install {self.cfg.appid}"
                    )
                    return False

            if "config" in stage:
                # Stage 6
                if Deploy.config(self) is not True:
                    self.progressbar(
                        error=True,
                        msg="Couldn't configure Application Hosting on IOS-XE",
                    )
                    return False

            if "activate" in stage:
                # Stage 7
                if Deploy.activate(self) is not True:
                    self.progressbar(
                        error=True, msg=f"Couldn't activate {self.cfg.appid}"
                    )
                    return False

            if "start" in stage:
                # Stage 8
                if Deploy.start(self) is not True:
                    self.progressbar(error=True, msg=f"Couldn't start {self.cfg.appid}")
                    return False

            # Stage 9
            self.progressbar(success=True, msg="Thousand Eyes Agent Deployed")

        return True

    def undeploy(self, host):
        """
        Parameters
        ----------
        host : str
            Catalyst 9000 host
        Returns
        -------
        bool
            If successful or failed
        """

        self.device_api = Netconf(
            host=host,
            username=self.cfg.username,
            password=self.cfg.password,
            port=self.cfg.port,
            timeout=self.cfg.timeout,
        )
        self.host = host
        self.mode = "undeploy"

        self.pbar = tqdm(self.stages[self.mode], desc=f"{self.host}: connecting")
        for stage in self.pbar:
            self.progressbar(stage=stage)

            if "stop" in stage:
                # Stage 0
                if Undeploy.stop(self) is not True:
                    self.progressbar(error=True, msg=f"Couldn't stop {self.cfg.appid}")
                    return False
            if "deactivate" in stage:
                # Stage 1
                if Undeploy.deactivate(self) is not True:
                    self.progressbar(
                        error=True, msg=f"Couldn't deactivate {self.cfg.appid}"
                    )
                    return False

            if "uninstall" in stage:
                # Stage 2
                if Undeploy.uninstall(self) is not True:
                    self.progressbar(
                        error=True, msg=f"Couldn't uninstall {self.cfg.appid}"
                    )
                    return False

            if "config" in stage:
                # Stage 3
                if Undeploy.config(self) is not True:
                    self.progressbar(
                        error=True,
                        msg="Couldn't remove configure for Application Hosting on IOS-XE",
                    )
                    return False

            if "disable" in stage:
                # Stage 4
                if Undeploy.iox(self) is not True:
                    self.progressbar(
                        error=True, msg="Couldn't disable IOX (container support)"
                    )
                    return False

            # Stage 5
            self.progressbar(success=True, msg="Thousand Eyes Agent Undeployed")

        return True

    def progressbar(self, success=False, error=False, stage=None, msg=None):
        """ Display progressbar """
        if error is True:
            self.pbar.set_description(f"{self.host}: {msg}")
            return
        if success is True:
            self.pbar.set_description(desc=f"{self.host}: {msg.capitalize()}")
            return
        self.pbar.set_description(desc=f"{self.host}: {stage.capitalize()}")
        return



class Metadata:
    name = "cat9kthousandeyesctl"
    author = "Robert Csapo"
    email = "rcsapo@cisco.com"
    description = "Cisco Catalyst 9000 with Thousand Eyes agents"
    repo_url = "https://github.com/robertcsapo/cat9kthousandeyesctl"
    version = "0.0.1"
    license = "CISCO SAMPLE CODE LICENSE"
    copyright = "Copyright (c) 2021 Cisco and/or its affiliates."



from os import path
from setuptools import setup, find_packages
from cat9kthousandeyesctl import Metadata

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

with open(
    path.join(path.abspath(path.dirname(__file__)), "README.md"), encoding="utf-8"
) as f:
    long_description = f.read()

setup(
    name=Metadata.name,
    author=Metadata.author,
    author_email=Metadata.email,
    description=Metadata.description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=Metadata.repo_url,
    version=Metadata.version,
    packages=find_packages(),
    py_modules=[Metadata.name],
    install_requires=requirements,
    entry_points=f"""
        [console_scripts]
        {Metadata.name}={Metadata.name}.main:cli
        """,
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Build Tools",
        "License :: Other/Proprietary License",
    ],
    license=Metadata.license,
    copyright=Metadata.copyright,
    python_requires=">=3.7",
)



"""
Configuration
"""
from dataclasses import dataclass, fields
import yaml


@dataclass(frozen=True)
class Config:
    """ Config for Thousand Eyes Agent """

    hosts: list
    username: str
    password: str
    port: int
    timeout: int
    download_url: str
    appid: str
    vlan: int
    token: str


class Supported:
    """ Supported Catalyst 9000 devices """

    hardware = ["C9300", "C9400"]
    subscriptions = ["dna-advantage"]
    versions = [
        "17.03.03",
        "17.04.",
        "17.05.",
    ]


class Configs:
    """
    Parameters
    ----------
    config : file
        YAML config
    Returns
    -------
    dataclass: Config
        Config for Thousand Eyes Agent
    """

    @classmethod
    def load(cls, config):
        """
        Load config from YAML
        ---
        Returns
        dataclass: Config
        """
        try:
            with open(config, "r") as file:
                cfg_dict = yaml.load(file, Loader=yaml.FullLoader)
        except Exception as error_msg:
            raise Exception(f"Can't read config file - {error_msg}")
        return cls.__validate(cfg_dict)

    @classmethod
    def interactive(cls, config):
        """
        Load config from prompt (input)
        ---
        Returns
        dataclass: Config
        """
        return cls.__validate(config)

    @classmethod
    def __validate(cls, cfg_dict):
        """
        Create instance from dataclass
        ---
        Returns
        dataclass: Config
        """

        cfg = Config(
            hosts=cfg_dict["hosts"],
            username=cfg_dict["username"],
            password=cfg_dict["password"],
            port=cfg_dict["port"],
            timeout=cfg_dict["timeout"],
            download_url=cfg_dict["download_url"],
            appid=cfg_dict["appid"],
            vlan=cfg_dict["vlan"],
            token=cfg_dict["token"],
        )

        return cfg
